<?php declare(strict_types=1);

namespace Albis\AlbisLease\Storefront\Controller;

use Albis\AlbisLease\Core\Checkout\Event\AlbisLeasingApplicationPlacedEvent;
use GuzzleHttp\Client;
use mysql_xdevapi\BaseResult;
use Shopware\Core\Checkout\Order\Aggregate\OrderAddress\OrderAddressEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderCustomer\OrderCustomerEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Content\Mail\Service\AbstractMailService;
use Shopware\Core\Content\Mail\Service\MailAttachmentsConfig;
use Shopware\Core\Content\MailTemplate\MailTemplateEntity;
use Shopware\Core\Content\MailTemplate\Subscriber\MailSendSubscriberConfig;
use Shopware\Core\Content\Product\ProductEntity;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Event\EventData\MailRecipientStruct;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Shopware\Storefront\Controller\StorefrontController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * @Route(defaults={"_routeScope"={"storefront"}})
 */
class AlbisLeasingController extends StorefrontController
{
    /**
     * @var EntityRepository
     */
    private $productRepository;

    /**
     * @var EntityRepository
     */
    private $orderRepository;

    /**
     * @var SystemConfigService
     */
    private $systemConfigService;

    /**
     * @var EventDispatcherInterface
     */
    private $eventDispatcher;

    /** @var OrderTransactionStateHandler */
    private $orderTransactionStateHandler;

    /** @var Client */
    private $client;

    /** @var array */
    private $albisConfig;
    private AbstractMailService $mailService;

    /**
     * AlbisLeasingController constructor.
     * @param EntityRepository $productRepository
     * @param EntityRepository $orderRepository
     * @param SystemConfigService $systemConfigService
     * @param EventDispatcherInterface $eventDispatcher
     * @param OrderTransactionStateHandler $stateHandler
     */
    public function __construct(
        EntityRepository $productRepository,
        EntityRepository $orderRepository,
        SystemConfigService $systemConfigService,
        EventDispatcherInterface $eventDispatcher,
        OrderTransactionStateHandler $stateHandler,
        AbstractMailService $mailService
    )
    {
        $this->productRepository = $productRepository;
        $this->orderRepository = $orderRepository;
        $this->systemConfigService = $systemConfigService;
        $this->eventDispatcher = $eventDispatcher;
        $this->orderTransactionStateHandler = $stateHandler;


        $this->client = new Client();
        $this->albisConfig = $this->systemConfigService->get('AlbisLease.config');
        $this->albisConfig['productGroup'] = rtrim($this->albisConfig['productGroup'], '0');
        $this->mailService = $mailService;
    }

    /**
     * @Route("/albis/{productnumber}", name="frontend.albis.modal", defaults={"XmlHttpRequest"=true}, options={"seo"="false"}, methods={"GET"})
     * @param Request $request
     * @param SalesChannelContext $context
     */
    public function albis(Request $request, SalesChannelContext $context)
    {
        $productNumber = $request->get('productnumber');

        $filter = new EqualsFilter('productNumber', $productNumber);

        $criteria = new Criteria();
        $criteria->addFilter($filter);

        $products = $this->productRepository->search($criteria, $context->getContext());

        /** @var ProductEntity $product */
        $product = $products->first();

        $response = $this->getAlbisLeasingRates($product->getPrice()->first()->getNet(), $context);

        if(array_key_exists('error', $response) && is_array($response['error'])) {
            if(array_key_exists('code', $response['error'])) {
                throw new \Exception("Error {$response['error']['code']}: {$response['error']['message']}");
            } else {
                throw new \Exception("Error: {$response['error']['message']}");
            }
        }

        return $this->renderStorefront('@AlbisLease/storefront/page/albis/rates-modal.html.twig', [
            'rates' => $response['result']['raten']
        ]);
    }

    /**
     * @Route("/checkout/albis/{orderId}/{transactionId}", name="frontend.checkout.albis", options={"seo"="false"}, methods={"GET"})
     * @param Request $request
     * @param SalesChannelContext $context
     */
    public function checkout(Request $request, SalesChannelContext  $context)
    {
        $orderId = $request->get('orderId');
        $transactionId = $request->get('transactionId');

        $criteria = new Criteria([
            $orderId
        ]);

        /** @var OrderEntity $order */
        $order = $this->orderRepository->search($criteria, $context->getContext())->first();

        $criteria = new Criteria([
            $transactionId
        ]);

        $transactionRepository = $this->container->get('order_transaction.repository');

        /** @var OrderTransactionEntity $transaction */
        $transaction = $transactionRepository->search($criteria, $context->getContext())->first();

        $technicalName = $transaction->getStateMachineState()->getTechnicalName();

        if($technicalName == 'in_progress') {
            return $this->redirectToRoute(
                'frontend.checkout.cart.page'
            );
        }

        /** @var OrderCustomerEntity $cust */
        $customer = $order->getOrderCustomer();

        /** @var EntityRepository $orderAddressRepository */
        $orderAddressRepository = $this->container->get('order_address.repository');

        /** @var OrderAddressEntity|null $billingAddress */
        $billingAddress = $orderAddressRepository->search(new Criteria([$order->getBillingAddressId()]), $context->getContext())->first();

        $companyTypes = $this->getAlbisCompanyTypes();

        $session = $request->getSession();

        // calculate max prepayment
        $orderAmount = $order->getAmountNet();
        $minLeasingAmount = 500;

        $prepayment = [
            'min' => floatval(0),
            'max' => floor(floatval($orderAmount - $minLeasingAmount)),
            'selected' => $session->get('albis_prepayment', 0)
        ];

        if($prepayment['selected'] > $prepayment['max']) {
            $this->addFlash('info', $this->trans(''));
            $this->addFlash('info', "albis.errors.prepaymentToHigh");
            $prepayment['selected'] = $prepayment['max'];
        }

        $rates = $this->getAlbisLeasingRates($order->getAmountNet(), $context, $prepayment['selected']);

        if(array_key_exists('error', $rates) && is_array($rates['error'])) {
            $this->addFlash('danger', $rates['error']['message']);

        } else {
            $rates = $rates['result']['raten'];
            $lastRate = end($rates);
        }

        $albisInit = $session->get('albis_initiated');
        if(!$albisInit) {

            // set selected rate to end
            $session->set('albis_rate_selected', $lastRate['laufzeit']);

            $session->set('albis_customer_company', $customer->getCompany());
            $session->set('albis_customer_street', $billingAddress->getStreet());
            $session->set('albis_customer_zipcode', $billingAddress->getZipcode());
            $session->set('albis_customer_city', $billingAddress->getCity());
            $session->set('albis_customer_email', $customer->getEmail());

            $session->set('albis_customer_salutation', $customer->getSalutation());
            $session->set('albis_customer_firstname', $customer->getFirstName());
            $session->set('albis_customer_lastname', $customer->getLastName());

            $session->set('albis_initiated', true);
        }

        return $this->renderStorefront('@AlbisLease/storefront/page/checkout/albis.html.twig', [
            'orderId' => $orderId,
            'transactionId' => $transactionId,
            'order' => $order,
            'rates' => $rates,
            'albis' => [
                'rate_selected' => $session->get('albis_rate_selected'),
                'prepayment' => $prepayment,

                'customer' => [
                    'company' => $session->get('albis_customer_company'),
                    'companyId' => $session->get('albis_customer_company_id'),
                    'street' => $session->get('albis_customer_street'),
                    'zipcode' => $session->get('albis_customer_zipcode'),
                    'city' => $session->get('albis_customer_city'),
                    'phone' => $session->get('albis_customer_phone'),
                    'email' => $session->get('albis_customer_email'),
                    'iban' => $session->get('albis_customer_iban'),

                    // CEO
                    'ceo' => [
                        'salutation' => $session->get('albis_customer_ceo_salutation'),
                        'firstname' => $session->get('albis_customer_ceo_firstname'),
                        'lastname' => $session->get('albis_customer_ceo_lastname'),
                        'birthday_day' => $session->get('albis_customer_ceo_birthday_day'),
                        'birthday_month' => $session->get('albis_customer_ceo_birthday_month'),
                        'birthday_year' => $session->get('albis_customer_ceo_birthday_year'),
                        'street' => $session->get('albis_customer_ceo_street'),
                        'zipcode' => $session->get('albis_customer_ceo_zipcode'),
                        'city' => $session->get('albis_customer_ceo_city'),
                        'phone' => $session->get('albis_customer_phone'),
                        'email' => $session->get('albis_customer_email'),
                    ]
                ],
                'tos' => [
                    'albis' => $session->get('albis_tos_albis'),
                ]
            ],
            'companyTypes' => $companyTypes
        ]);
    }

    /**
     * @Route("/checkout/albis/prepayment/{orderId}/{transactionId}", name="frontend.checkout.albis.prepayment", options={"seo"="false"}, methods={"POST"})
     * @param Request $request
     * @param SalesChannelContext $context
     */
    public function addPrepayment(Request $request, SalesChannelContext  $context)
    {
        $orderId = $request->get('orderId');
        $transactionId = $request->get('transactionId');
        $session = $request->getSession();
        $post = $request->request->all();

        $session->set('albis_prepayment', floatval($post['prepayment']));

        return $this->redirectToRoute(
            'frontend.checkout.albis',
            [
                'orderId' => $orderId,
                'transactionId' => $transactionId,
            ]
        );
    }

    /**
     * @Route("/checkout/albis/confirm/{orderId}/{transactionId}", name="frontend.checkout.albis.confirm", options={"seo"="false"}, methods={"POST"})
     * @param Request $request
     * @param SalesChannelContext $context
     */
    public function confirm(Request $request, SalesChannelContext $context)
    {
        $orderId = $request->get('orderId');
        $transactionId = $request->get('transactionId');

        $session = $request->getSession();

        $criteria = new Criteria([
            $orderId
        ]);
        $criteria->addAssociation('lineItems');

        /** @var OrderEntity $order */
        $order = $this->orderRepository->search($criteria, $context->getContext())->first();

        if(!$order) {
            return $this->redirectWithError($orderId, $transactionId, $this->trans('albis.errors.orderNotFound'));
        }

        $post = $request->request->all();

        if($post['rate']) $session->set('albis_rate_selected', trim($post['rate']));

        if(!$post['rate']) return $this->redirectWithError($orderId, $transactionId, $this->trans('albis.errors.missingRate'));

        // set new values to session
        if($post['company']) $session->set('albis_customer_company', trim($post['company']));
        if($post['company_id']) $session->set('albis_customer_company_id', trim($post['company_id']));
        if($post['street']) $session->set('albis_customer_street', trim($post['street']));
        if($post['zipcode']) $session->set('albis_customer_zipcode', trim($post['zipcode']));
        if($post['city']) $session->set('albis_customer_city', trim($post['city']));
        if($post['email']) $session->set('albis_customer_email', trim($post['email']));
        if($post['phone']) $session->set('albis_customer_phone', trim($post['phone']));
        if($post['iban']) $session->set('albis_customer_iban', trim($post['iban']));

        if($request->request->get('ceo_salutation')) $session->set('albis_customer_ceo_salutation', trim($post['ceo_salutation']));
        if($request->request->get('ceo_firstname')) $session->set('albis_customer_ceo_firstname', trim($post['ceo_firstname']));
        if($request->request->get('ceo_lastname')) $session->set('albis_customer_ceo_lastname', trim($post['ceo_lastname']));

        if(isset($post['ceo_birthday_day'])) $session->set('albis_customer_ceo_birthday_day', trim($post['ceo_birthday_day']));
        if(isset($post['ceo_birthday_month'])) $session->set('albis_customer_ceo_birthday_month', trim($post['ceo_birthday_month']));
        if(isset($post['ceo_birthday_year'])) $session->set('albis_customer_ceo_birthday_year', trim($post['ceo_birthday_year']));

        if($request->request->get('ceo_street')) $session->set('albis_customer_ceo_street', trim($post['ceo_street']));
        if($request->request->get('ceo_zipcode')) $session->set('albis_customer_ceo_zipcode', trim($post['ceo_zipcode']));
        if($request->request->get('ceo_city')) $session->set('albis_customer_ceo_city', trim($post['ceo_city']));
        if($request->request->get('ceo_phone')) $session->set('albis_customer_ceo_phone', trim($post['ceo_phone']));
        if($request->request->get('ceo_email')) $session->set('albis_customer_ceo_email', trim($post['ceo_email']));

        if($post['tos_albis']) $session->set('albis_tos_albis', 1);

        $client = new Client();
        $requestId = sha1(microtime());

        $salesChannelId = $context->getContext()->getSource()->getSalesChannelId();
        $albisConfig = $this->systemConfigService->get('AlbisLease.config', $salesChannelId);

        $albisContract = 2;
        $albisUsername = $albisConfig['usernameContract2'] ?? '';
        $albisPassword = $albisConfig['passwordContract2'] ?? '';
        foreach ($order->getLineItems() as $lineItem) {
            $payload = $lineItem->getPayload();
            $customFields = $payload['customFields'];
            if (isset($customFields['custom_albis_contract']) && $customFields['custom_albis_contract'] === 1) {
                $albisContract = 1;
                $albisUsername = $albisConfig['username'];
                $albisPassword = $albisConfig['password'];
            }
        }

        $response = $this->getAlbisLeasingRates($order->getAmountNet(), $context);

        if(!array_key_exists('result', $response) || !array_key_exists('raten', $response['result'])) {
            return $this->redirectWithError($orderId, $transactionId, $this->trans('albis.errors.missingResponse'));
        }

        $rateKey = array_search($post['rate'], array_column($response['result']['raten'], 'laufzeit'));

        if($rateKey === false) {
            return $this->redirectWithError($orderId, $transactionId, $this->trans('albis.errors.missingRate'));
        }

        $rateMonthly = $response['result']['raten'][$rateKey]['rate'];

        // check if ceo data is filled
        $errors = [];

        if($order->getAmountNet() >= $albisConfig['thresholdCeoData']) {

            if(!isset($post['ceo_birthday_year'])) $errors[] = $this->trans('albis.errors.missingCeoBirthdayYear');
            if(!isset($post['ceo_birthday_month'])) $errors[] = $this->trans('albis.errors.missingCeoBirthdayMonth');
            if(!isset($post['ceo_birthday_day'])) $errors[] = $this->trans('albis.errors.missingCeoBirthdayDay');
        }

        if($errors) {
            return $this->redirectWithError($orderId, $transactionId, $errors);
        }

        /** @var EntityRepositoryInterface $productGroup */
        $productGroupRepository = $this->container->get('albis_product_group.repository');

        $productGroup = $productGroupRepository->search(
            (new Criteria())->addFilter(new EqualsFilter('id', $this->albisConfig['productGroup'])),
            $context->getContext()
        )->first();

        $response = $client->post($albisConfig['endpoint'], [
            'json' => [
                'id' => $requestId,
                'method' => 'stelleAntrag',
                'params' => [
                    $albisUsername,
                    $albisPassword,
                    [
                        'objekt' => 'Shopware Bestellung ' . $order->getOrderNumber(),
                        'kaufpreis' => $order->getAmountNet(),
                        'mietsz' => $session->get('albis_prepayment', 0), // Sonderzahlung
                        'laufzeit' => $post['rate'],
                        'rate' => $rateMonthly,
                        'leasingnehmer' => [
                            'name' => $post['company'],
                            'strasse' => $post['street'],
                            'plz' => $post['zipcode'],
                            'ort' => $post['city'],
                            'geschaeftsfuehrer' => [
                                'anrede' => $post['ceo_salutation'] ?: null,
                                'vorname' => $post['ceo_firstname'] ?: null,
                                'nachname' => $post['ceo_lastname'] ?: null,
                                'strasse' => $post['ceo_street'] ?: null,
                                'plz' => $post['ceo_zipcode'] ?: null,
                                'ort' => $post['ceo_city'] ?: null,
                                'gebdat' => $post['ceo_birthday_year'] . '-' . sprintf('%02d', $post['ceo_birthday_month']) . '-' . sprintf('%02d', $post['ceo_birthday_day']),
                            ],
                            'rechtsform' => $post['company_id'],
                            'telefon' => $post['phone'],
                            'email' => $post['email'],
                        ],
                        'provision' => $albisConfig['provision'],
                        'vertragsart' => $albisContract,
                        'prodgrp' => $productGroup->getAlbisId(),
                        'zahlweise' => 2,
                        'iban' => $post['iban'],
                        'vertrag_an_ln' => true
                    ]
                ]
            ]
        ]);

        $response = json_decode($response->getBody()->getContents(), true);

        if($response['error']) {
            $this->addFlash('danger', $response['error']['code'] .': ' . $response['error']['message']);
            return $this->redirectToRoute(
                'frontend.checkout.albis',
                [
                    'orderId' => $orderId,
                    'transactionId' => $transactionId
                ]
            );
        }

        $albis = [
            'shopware' => [
                'customer' => [
                    'firstname' => $session->get('albis_customer_firstname'),
                    'lastname' => $session->get('albis_customer_lastname'),
                ],
                'order' => [
                  'number' => $order->getOrderNumber()
                ],
            ],
            'application' => [
                'number' => $response['result'],
                'amount' => $order->getAmountNet(),
                'prepayment' => $session->get('albis_prepayment', 0),
                'duration' => $post['rate'],
                'amountMonthly' => $rateMonthly,
                'customer' => [
                    'name' => $post['company'],
                    'street' => $post['street'],
                    'zipcode' => $post['zipcode'],
                    'city' => $post['city'],
                    'phone' => $post['phone'],
                    'email' => $post['email'],
                    'iban' => $post['iban'],
                ],
                'ceo' => [
                    'salutation' => $post['ceo_salutation'],
                    'firstname' => $post['ceo_firstname'],
                    'lastname' => $post['ceo_lastname'],
                    'birthday' => [
                        'day' => isset($post['ceo_birthday_day']) ? $post['ceo_birthday_day'] : null,
                        'month' => isset($post['ceo_birthday_month']) ? $post['ceo_birthday_month'] : null,
                        'year' => isset($post['ceo_birthday_year']) ? $post['ceo_birthday_year'] : null,
                    ],
                    'street' => $post['ceo_street']?? '',
                    'zipcode' => $post['ceo_zipcode']?? '',
                    'city' => $post['ceo_city']?? '',
                    'phone' => $post['ceo_phone'] ?? '',
                    'email' => $post['ceo_email']?? '',
                ]
            ]
        ];

        /** @var EntityRepository $mailTemplateTypeRepository */
        $mailTemplateTypeRepository = $this->container->get('mail_template_type.repository');
        /** @var EntityRepository $mailTemplateRepository */
        $mailTemplateRepository = $this->container->get('mail_template.repository');

        $myCustomMailTemplateType = $mailTemplateTypeRepository->search(
            (new Criteria())->addFilter(new EqualsFilter('technicalName', 'albis_contract_submit')),
            $context->getContext()
        )->first();

        $mailTemplate = $mailTemplateRepository->search(
            (new Criteria())->addFilter(new EqualsFilter('mailTemplateTypeId', $myCustomMailTemplateType->getId())),
            $context->getContext()
        )->first();

        $data = [];
        $data['mailTemplateData']['albis'] = $albis;
        $data['recipients'] = [
            $order->getOrderCustomer()->getEmail() => $order->getOrderCustomer()->getFirstName().' '.$order->getOrderCustomer()->getLastName(),
            ];
        $data['salesChannelId'] = $context->getSalesChannelId();
        $data['contentHtml'] = $mailTemplate->getContentHtml();
        $data['contentPlain'] = $mailTemplate->getContentPlain();
        $data['subject'] = $mailTemplate->getSubject();
        $data['senderName'] = $mailTemplate->getSenderName();
        $mailTemplateData = $data['mailTemplateData'] ?? [];
        $extension = new MailSendSubscriberConfig(
            false,
            $data['documentIds'] ?? [],
            $data['mediaIds'] ?? [],
        );

        $data['attachmentsConfig'] = new MailAttachmentsConfig(
            $context->getContext(),
            new MailTemplateEntity(),
            $extension,
            [],
            $mailTemplateData['order']['id'] ?? null,
        );

        $this->mailService->send($data, $context->getContext(), $mailTemplateData);

        $this->orderTransactionStateHandler->process($transactionId, $context->getContext());

        return $this->redirectToRoute(
            'frontend.checkout.finish.page',
            [
                'orderId' => $order->getId(),
            ]
        );
    }

    /*
     * API Helper methods
     */
    private function getRequestId()
    {
        return sha1(microtime());
    }

    private function getAlbisCompanyTypes()
    {
        $response = $this->client->post($this->albisConfig['endpoint'], [
            'json' => [
                'id' => $this->getRequestId(),
                'method' => 'getRechtsformen',
                'params' => [
                    $this->albisConfig['username'],
                    $this->albisConfig['password'],
                ]
            ]
        ]);

        $companyTypes = json_decode($response->getBody()->getContents(), true);

        return $companyTypes['result'];
    }

    private function getAlbisLeasingRates(
        float $amountTotal,
        SalesChannelContext $context,
        float $prepayment = 0
    ) {
        /** @var EntityRepositoryInterface $productGroup */
        $productGroupRepository = $this->container->get('albis_product_group.repository');

        $productGroup = $productGroupRepository->search(
            (new Criteria())->addFilter(new EqualsFilter('id', $this->albisConfig['productGroup'])),
            $context->getContext()
        )->first();

        $response = $this->client->post($this->albisConfig['endpoint'], [
            'json' => [
                'id' => $this->getRequestId(),
                'method' => 'getRate',
                'params' => [
                    $this->albisConfig['username'],
                    $this->albisConfig['password'],
                    [
                        'kaufpreis' => $amountTotal,
                        'mietsz' => $prepayment,
                        'prodgrp' => $productGroup->getAlbisId(),
                        'vertragsart' => 1,
                        'zahlweise' => 2,
                        'provision' => $this->albisConfig['provision']
                    ]
                ]
            ]
        ]);

        return json_decode($response->getBody()->getContents(), true);
    }

    private function redirectWithError($orderId, $transactionId, $message)
    {
        if(is_array($message)) {
            foreach($message as $item) {
                $this->addFlash('danger', $item);
            }
        } else {
            $this->addFlash('danger', $message);
        }


        return $this->redirectToRoute(
            'frontend.checkout.albis',
            [
                'orderId' => $orderId,
                'transactionId' => $transactionId
            ]
        );
    }
}